@@ -307,8 +307,7 @@ static void vchan_play_callback(pa_mainloop_api *UNUSED(a),
     }
 
     /* process playback data */
-    if (u->play_stream && pa_stream_get_state(u->play_stream) == PA_STREAM_READY)
-        process_playback_data(u, u->play_stream, pa_stream_writable_size(u->play_stream));
+    process_playback_data(u, u->play_stream, pa_stream_writable_size(u->play_stream));
 }
 
 static void vchan_rec_callback(pa_mainloop_api *UNUSED(a),
@@ -325,52 +324,50 @@ static void vchan_rec_callback(pa_mainloop_api *UNUSED(a),
         return;
     }
 
-    if (u->rec_stream && pa_stream_get_state(u->rec_stream) == PA_STREAM_READY) {
-        /* process VM control command */
-        uint32_t cmd;
-        while (libvchan_data_ready(u->rec_ctrl) >= (int)sizeof(cmd)) {
-            if (libvchan_read(u->rec_ctrl, (char*)&cmd, sizeof(cmd)) != sizeof(cmd)) {
-                if (!pa_stream_is_corked(u->rec_stream))
+    /* process VM control command */
+    uint32_t cmd;
+    while (libvchan_data_ready(u->rec_ctrl) >= (int)sizeof(cmd)) {
+        if (libvchan_read(u->rec_ctrl, (char*)&cmd, sizeof(cmd)) != sizeof(cmd)) {
+            if (!pa_stream_is_corked(u->rec_stream))
+                pa_stream_cork(u->rec_stream, 1, NULL, u);
+            fprintf(stderr, "Failed to read from vchan\n");
+            quit(u, 1);
+            return;
+        }
+        switch (cmd) {
+            case QUBES_PA_SOURCE_START_CMD:
+                g_mutex_lock(&u->prop_mutex);
+                u->rec_requested = 1;
+                if (u->rec_allowed) {
+                    pacat_log("Recording start");
+                    pa_stream_cork(u->rec_stream, 0, NULL, u);
+                } else
+                    pacat_log("Recording requested but not allowed");
+                g_mutex_unlock(&u->prop_mutex);
+                break;
+            case QUBES_PA_SOURCE_STOP_CMD:
+                g_mutex_lock(&u->prop_mutex);
+                u->rec_requested = 0;
+                if (!pa_stream_is_corked(u->rec_stream)) {
+                    pacat_log("Recording stop");
                     pa_stream_cork(u->rec_stream, 1, NULL, u);
-                fprintf(stderr, "Failed to read from vchan\n");
-                quit(u, 1);
-                return;
-            }
-            switch (cmd) {
-                case QUBES_PA_SOURCE_START_CMD:
-                    g_mutex_lock(&u->prop_mutex);
-                    u->rec_requested = 1;
-                    if (u->rec_allowed) {
-                        pacat_log("Recording start");
-                        pa_stream_cork(u->rec_stream, 0, NULL, u);
-                    } else
-                        pacat_log("Recording requested but not allowed");
-                    g_mutex_unlock(&u->prop_mutex);
-                    break;
-                case QUBES_PA_SOURCE_STOP_CMD:
-                    g_mutex_lock(&u->prop_mutex);
-                    u->rec_requested = 0;
-                    if (!pa_stream_is_corked(u->rec_stream)) {
-                        pacat_log("Recording stop");
-                        pa_stream_cork(u->rec_stream, 1, NULL, u);
-                    }
-                    g_mutex_unlock(&u->prop_mutex);
-                    break;
-                case QUBES_PA_SINK_CORK_CMD:
-                    pacat_log("Stream cork");
-                    pa_stream_cork(u->play_stream, 1, NULL, u);
-                    break;
-                case QUBES_PA_SINK_UNCORK_CMD:
-                    pacat_log("Stream uncork");
-                    pa_stream_cork(u->play_stream, 0, NULL, u);
-                    break;
-            }
-        }
-        /* send the data if space is available */
-        if (libvchan_buffer_space(u->rec_ctrl)) {
-            send_rec_data(u->rec_stream, u);
+                }
+                g_mutex_unlock(&u->prop_mutex);
+                break;
+            case QUBES_PA_SINK_CORK_CMD:
+                pacat_log("Stream cork");
+                pa_stream_cork(u->play_stream, 1, NULL, u);
+                break;
+            case QUBES_PA_SINK_UNCORK_CMD:
+                pacat_log("Stream uncork");
+                pa_stream_cork(u->play_stream, 0, NULL, u);
+                break;
         }
     }
+    /* send the data if space is available */
+    if (libvchan_buffer_space(u->rec_ctrl)) {
+        send_rec_data(u->rec_stream, u);
+    }
 }
 
 /* This routine is called whenever the stream state changes */
@@ -380,7 +377,18 @@ static void stream_state_callback(pa_stream *s, void *userdata) {
 
     switch (pa_stream_get_state(s)) {
         case PA_STREAM_CREATING:
+	    break;
         case PA_STREAM_TERMINATED:
+	    if (u->play_stdio_event && u->play_stream == s) {
+               pacat_log("play stream terminated");
+		assert(u->mainloop_api);
+		u->mainloop_api->io_free(u->play_stdio_event);
+	    }
+	    if (u->rec_stdio_event && u->rec_stream == s) {
+               pacat_log("rec stream terminated");
+		assert(u->mainloop_api);
+		u->mainloop_api->io_free(u->rec_stdio_event);
+	    }
             break;
 
         case PA_STREAM_READY:
@@ -406,7 +414,22 @@ static void stream_state_callback(pa_stream *s, void *userdata) {
                         pa_stream_get_device_index(s),
                         pa_stream_is_suspended(s) ? "" : "not ");
             }
-
+	    if (u->play_stream == s) {
+		u->play_stdio_event = u->mainloop_api->io_new(u->mainloop_api,
+                    libvchan_fd_for_select(u->play_ctrl), PA_IO_EVENT_INPUT, vchan_play_callback,  u);
+		if (!u->play_stdio_event) {
+		    pacat_log("io_new play failed");
+		    quit(u, 1);
+		}
+	    }
+	    if (u->rec_stream == s) {
+		u->rec_stdio_event = u->mainloop_api->io_new(u->mainloop_api,
+			libvchan_fd_for_select(u->rec_ctrl), PA_IO_EVENT_INPUT, vchan_rec_callback, u);
+		if (!u->rec_stdio_event) {
+		    pacat_log("io_new rec failed");
+		    quit(u, 1);
+		}
+	    }
             break;
 
         case PA_STREAM_FAILED:
@@ -891,20 +914,6 @@ int main(int argc, char *argv[])
 
     u.mainloop_api = pa_glib_mainloop_get_api(m);
 
-    u.play_stdio_event = u.mainloop_api->io_new(u.mainloop_api,
-            libvchan_fd_for_select(u.play_ctrl), PA_IO_EVENT_INPUT, vchan_play_callback, &u);
-    if (!u.play_stdio_event) {
-        pacat_log("io_new play failed");
-        goto quit;
-    }
-
-    u.rec_stdio_event = u.mainloop_api->io_new(u.mainloop_api,
-            libvchan_fd_for_select(u.rec_ctrl), PA_IO_EVENT_INPUT, vchan_rec_callback, &u);
-    if (!u.rec_stdio_event) {
-        pacat_log("io_new rec failed");
-        goto quit;
-    }
-
     pa_gettimeofday(&tv);
     pa_timeval_add(&tv, (pa_usec_t) 5 * 1000 * PA_USEC_PER_MSEC);
     time_event = u.mainloop_api->time_new(u.mainloop_api, &tv, check_vchan_eof_timer, &u);
@@ -949,15 +958,6 @@ quit:
     if (u.context)
         pa_context_unref(u.context);
 
-    if (u.play_stdio_event) {
-        assert(u.mainloop_api);
-        u.mainloop_api->io_free(u.play_stdio_event);
-    }
-    if (u.rec_stdio_event) {
-        assert(u.mainloop_api);
-        u.mainloop_api->io_free(u.rec_stdio_event);
-    }
-
     if (time_event) {
         assert(u.mainloop_api);
         u.mainloop_api->time_free(time_event);
